use lettre::message::Mailbox;
use lettre::{AsyncSmtpTransport, AsyncTransport, Message, Tokio1Executor};

use crate::domain::{SubscriberEmail, SubscriberName};

// here I use my own email server instead of an API
// I skip the "properly call and test/mock and API" part of the book
// But I learn how to directly send mail from rust

pub struct EmailClient {
    pub sender: Mailbox,
    pub mailer: AsyncSmtpTransport<Tokio1Executor>,
}

impl EmailClient {
    pub fn format_recipient(name: SubscriberName, email: SubscriberEmail) -> Mailbox {
        format!("{} <{}>", name.as_ref(), email.as_ref())
            .parse()
            .unwrap()
    }

    pub async fn send_email(
        &self,
        recipient: Mailbox,
        subject: &str,
        html_content: &str,
    ) -> Result<(), String> {
        let email = Message::builder()
            .from(self.sender.to_owned())
            .to(recipient)
            .subject(subject)
            .body(String::from(html_content))
            .unwrap();
        match self.mailer.send(email).await {
            Ok(_) => Ok(()),
            Err(e) => Err(format!("Could not send email: {:?}", e)),
        }
    }
}

#[cfg(test)]
mod tests {
    use crate::{
        configuration::parse_email_config,
        domain::{SubscriberEmail, SubscriberName},
    };

    use super::EmailClient;

    #[tokio::test]
    async fn send_email_without_error() {
        let email_client = parse_email_config().email_client();
        let name = SubscriberName::parse("flavien".to_owned()).unwrap();
        let email = SubscriberEmail::parse("flavien@coincoinmail.fr".to_owned()).unwrap();
        let recipient = EmailClient::format_recipient(name, email);
        email_client
            .send_email(recipient, "Zero2Prod test", "OK")
            .await
            .expect("Mail sending error");
    }
}
