use clap::{command, Parser};
use std::net::TcpListener;
use zero2prod::configuration::{parse_db_config, parse_email_config};
use zero2prod::startup::run;
use zero2prod::telemetry::{get_subscriber, init_subscriber};

/// Zero2prod newsletter server
#[derive(Parser, Debug)]
#[command(author, version, about)]
struct Args {
    /// Port to listen
    #[arg(short, long, default_value_t = 8000)]
    port: u32,
}

#[tokio::main]
async fn main() -> std::io::Result<()> {
    let subscriber = get_subscriber("zero2prod".into(), "info".into(), std::io::stdout);
    init_subscriber(subscriber);

    let args = Args::parse();
    let address = match std::env::var("PROD").is_ok() {
        true => format!("0.0.0.0:{}", args.port),
        false => format!("127.0.0.1:{}", args.port),
    };
    let listener = TcpListener::bind(address)?;
    let connection_pool = parse_db_config().connection_pool().await;
    let email_client = parse_email_config().email_client();

    run(listener, connection_pool, email_client)?.await
}
